from zen_dash.objects.flex_data import FlexData
import warnings

warnings.warn("""
from zen_dash import flex_data is depricated, Please use
from zen_dash.objects import flex_data
""", DeprecationWarning)